# -*- coding: iso-8859-15 -*-

##############################################################################
#
# Copyright (c) 2006 Ingeniweb SAS
#
# This software is subject to the provisions of the GNU General Public License,
# Version 2.0 (GPL).  A copy of the GPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Plugins for zLOG

$Id: LogHandlers.py,v 1.2 2006/05/02 08:34:35 glenfant Exp $
"""

# Python standard distro
import os
from logging.handlers import RotatingFileHandler as BaseHandler

# Zope specific
from ZConfig.components.logger.handlers import HandlerFactory

# Reading and understanding this code requires you understand how
# zConfig.components.logger works

##
# Simple rotate log file handler and factory
##

class RotateFileHandler(BaseHandler):
    """A standard RotatingFileHandler with zLOG compatibility"""
    
    def __init__(self, path, max_bytes, backup_count):
        """Built by the factory and nothing else..."""
        
        filename =  os.path.abspath(path)
        BaseHandler.__init__(self, filename, mode='a', maxBytes=max_bytes,
                             backupCount=backup_count)
        self.baseFilename = filename
        return


    def reopen(self):
        """Mandatory for the factory but useless in our case"""
        
        self.close()
        self.stream = open(self.baseFilename, 'a')
        return


class RotateFileHandlerFactory(HandlerFactory):
    """Our factory referenced from the component.xml"""

    def create_loghandler(self):
        """Mandatory override"""

        return RotateFileHandler(self.section.path, self.section.max_bytes,
                                 self.section.backup_count)

##
# Rotate/zipping log file handler and factory
##

import zipfile

class RotateZipFileHandler(RotateFileHandler):
    """Rotates and zips the old log files"""

    def doRollover(self):
        """Overrides method from logging.handlers.RotatingFileHandler"""

        self.stream.close()
        if self.backupCount > 0:
            for i in range(self.backupCount - 1, 0, -1):
                sfn = "%s.%d.zip" % (self.baseFilename, i)
                dfn = "%s.%d.zip" % (self.baseFilename, i + 1)
                if os.path.exists(sfn):
                    if os.path.exists(dfn):
                        os.remove(dfn)
                    os.rename(sfn, dfn)
            dfn = self.baseFilename + ".1.zip"
            if os.path.exists(dfn):
                os.remove(dfn)
            zf = zipfile.ZipFile(dfn, 'w', zipfile.ZIP_DEFLATED)
            zf.write(self.baseFilename, os.path.basename(self.baseFilename))
            zf.close()
            os.remove(self.baseFilename)
        self.stream = open(self.baseFilename, "w")


class RotateZipFileHandlerFactory(HandlerFactory):
    """Our factory referenced from the component.xml"""

    def create_loghandler(self):
        """Mandatory override"""

        return RotateZipFileHandler(self.section.path, self.section.max_bytes,
                                    self.section.backup_count)
